"""Pytest of report model."""
import numpy as np
import pandas as pd
import pytest

from spectrafit.report import RegressionMetrics


class TestRegressionMetrics:
    """Test of the regression metrics module."""

    def test_raise_error(self) -> None:
        """Testing raise error."""
        with pytest.raises(ValueError) as excinfo:
            _ = RegressionMetrics(
                pd.DataFrame(
                    {
                        "intensity_0": np.random.rand(10),
                        "intensity_1": np.random.rand(10),
                        "fit_0": np.random.rand(10),
                    }
                )
            )
        assert "The shape of the real and fit data-values are not equal!" in str(
            excinfo.value
        )



"""Test of the converter plugin."""

import json

from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Any

import pytest
import tomli
import tomli_w
import yaml

from spectrafit.plugins.converter import convert


def test_cmd_converter(script_runner: Any) -> None:
    """Test the converter plugin."""
    ret = script_runner.run("spectrafit-converter", "-h")

    assert ret.success
    assert "Converter for 'SpectraFit' input and output files." in ret.stdout
    assert ret.stderr == ""


def test_raise_input_output() -> None:
    """Test raise error input format is similar to ouptut."""
    args = {
        "infile": Path("tests/data/input/input_1.yaml"),
        "format": "yaml",
    }
    with pytest.raises(ValueError) as excinfo:
        convert(args)

    assert (
        "The input file suffix 'yaml' is similar to the output file format 'yaml'."
        in str(excinfo.value)
    )


def test_raise_no_guilty_ouput() -> None:
    """Test illegal output format."""
    args = {
        "infile": Path("tests/data/input/input_1.yaml"),
        "format": "illegal",
    }
    with pytest.raises(ValueError) as excinfo:
        convert(args)
    assert "The output file format 'illegal' is not supported." in str(excinfo.value)


def test_json_conversion() -> None:
    """Test json to yaml conversion."""
    with TemporaryDirectory() as tmpdir:
        infile = Path(tmpdir) / "input_1.json"

        # write input json
        with open(infile, "w", encoding="utf8") as f:
            json.dump({"a": 1, "b": 2}, f)
        args = {
            "infile": infile,
            "format": "yaml",
        }
        convert(args)
        with open(infile.with_suffix(".yaml"), encoding="utf8") as f:
            data = yaml.safe_load(f)

        assert data == {"a": 1, "b": 2}


def test_yaml_conversion() -> None:
    """Test yaml to json conversion."""
    with TemporaryDirectory() as tmpdir:
        infile = Path(tmpdir) / "input_1.yaml"

        # write input yaml
        with open(infile, "w", encoding="utf8") as f:
            yaml.dump({"a": 1, "b": 2}, f)
        args = {
            "infile": infile,
            "format": "toml",
        }
        convert(args)
        with open(infile.with_suffix(".toml"), "rb") as f:
            data = tomli.load(f)

        assert data == {"a": 1, "b": 2}


def test_toml_conversion() -> None:
    """Test toml to json conversion."""
    with TemporaryDirectory() as tmpdir:
        infile = Path(tmpdir) / "input_1.toml"

        # write input toml
        with open(infile, "wb+") as f:
            tomli_w.dump({"a": 1, "b": 2}, f)
        args = {
            "infile": infile,
            "format": "json",
        }
        convert(args)
        with open(infile.with_suffix(".json"), encoding="utf8") as f:
            data = json.load(f)

        assert data == {"a": 1, "b": 2}



"""Test of the jupyter plugin."""

import sys

from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Any
from typing import Dict
from typing import List
from unittest import mock

import pandas as pd
import pytest

from spectrafit.api.cmd_model import DescriptionAPI
from spectrafit.api.notebook_model import FnameAPI
from spectrafit.api.notebook_model import PlotAPI
from spectrafit.api.report_model import InputAPI
from spectrafit.api.report_model import OutputAPI
from spectrafit.api.report_model import SolverAPI
from spectrafit.api.tools_model import DataPreProcessingAPI
from spectrafit.plugins.notebook import DataFrameDisplay
from spectrafit.plugins.notebook import DataFramePlot
from spectrafit.plugins.notebook import ExportReport
from spectrafit.plugins.notebook import ExportResults
from spectrafit.plugins.notebook import SpectraFitNotebook


__plotly_io_show__ = "plotly.io.show"


@pytest.fixture(name="dataframe")
def dataframe_fixture() -> pd.DataFrame:
    """Create a DataFrameDisplay object."""
    return pd.read_csv(
        "https://raw.githubusercontent.com/Anselmoo/spectrafit/main/Examples/data.csv"
    )


@pytest.fixture(name="dataframe_2")
def dataframe_2_fixture() -> pd.DataFrame:
    """Create a DataFrameDisplay object."""
    return pd.read_csv(
        "https://raw.githubusercontent.com/Anselmoo/"
        "spectrafit/main/Examples/example_1_fit.csv"
    )


@pytest.fixture(name="x_column")
def x_column_fixture(dataframe: pd.DataFrame) -> str:
    """Create a x_column object."""
    return str(dataframe.columns[1])


@pytest.fixture(name="y_column")
def y_column_fixture(dataframe: pd.DataFrame) -> str:
    """Create a y_column object."""
    return str(dataframe.columns[2])


@pytest.fixture(name="y_columns")
def y_columns_fixture(dataframe: pd.DataFrame) -> List[str]:
    """Create a y_column object."""
    return [str(dataframe.columns[2]), str(dataframe.columns[3])]


@pytest.fixture(name="initial_model")
def initial_model_fixture() -> List[Dict[str, Dict[str, Dict[str, Any]]]]:
    """Create a DataFrameDisplay object."""
    return [
        {
            "pseudovoigt": {
                "amplitude": {"max": 2, "min": 0, "vary": True, "value": 1},
                "center": {"max": 2, "min": -2, "vary": True, "value": 0},
                "fwhmg": {"max": 0.1, "min": 0.02, "vary": True, "value": 0.01},
                "fwhml": {"max": 0.1, "min": 0.01, "vary": True, "value": 0.01},
            }
        },
        {
            "gaussian": {
                "amplitude": {"max": 1.2, "min": 0.5, "vary": True, "value": 0.3},
                "center": {"max": 4, "min": 2, "vary": True, "value": 3.5},
                "fwhmg": {"max": 0.1, "min": 0.02, "vary": False, "value": 1.3},
            }
        },
    ]


@pytest.fixture(name="args_out")
def args_out_fixture() -> Dict[str, Any]:
    """Create a DataFrameDisplay object."""
    return {
        "global_": False,
        "fit_insights": {
            "correlations": {"a": {"a": 1, "b": 2}, "b": {"a": 2, "b": 3}},
            "configurations": {"n_components": 2, "n_iter": 1000},
            "covariance_matrix": {"a": {"a": 1, "b": 2}, "b": {"a": 2, "b": 3}},
            "statistics": {"aic": True, "bic": True, "aicc": True},
            "variables": {"a": {"value": 1, "std": 2}, "b": {"value": 3, "std": 4}},
            "errorbars": {
                "error_a": {"value": 1, "std": 2},
                "error_b": {"value": 3, "std": 4},
            },
        },
        "descriptive_statistic": pd.DataFrame({"mean": [1, 2], "std": [1, 2]}).to_dict(
            orient="list"
        ),
        "linear_correlation": pd.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]})
        .corr()
        .to_dict(orient="list"),
        "regression_metrics": {"MSE": [1], "RMSE": [1], "R2": [1]},
        "conf_interval": {"alpha": 0.05, "max_nfev": 100000},
        "confidence_interval": {"val_1": 2, "val_2": 3},
    }


@pytest.fixture(name="export_report")
def export_report_fixture(
    initial_model: List[Dict[str, Dict[str, Dict[str, Any]]]],
    args_out: Dict[str, Any],
    dataframe: pd.DataFrame,
    dataframe_2: pd.DataFrame,
) -> Dict[Any, Any]:
    """Create a ExportReport object."""
    return {
        "description": DescriptionAPI(),
        "initial_model": initial_model,
        "pre_processing": DataPreProcessingAPI(),
        "fname": FnameAPI(fname="test", suffix="out"),
        "args_out": args_out,
        "df_org": dataframe,
        "df_fit": dataframe_2,
        "df_pre": dataframe,
    }


@pytest.fixture(name="class_spectrafit")
def class_spectrafit_fixture(
    dataframe_2: pd.DataFrame, initial_model: List[Dict[str, Dict[str, Dict[str, Any]]]]
) -> Dict[Any, Any]:
    """Create a SpectraFitNotebook object."""
    with TemporaryDirectory() as tmpdir:
        _df = pd.DataFrame(data={"x": [1, 2, 3], "y": [1, 2, 3]})
        sp = SpectraFitNotebook(
            df=_df,
            x_column="x",
            y_column="y",
            folder=str(tmpdir),
            fname="test",
        )
        sp.df_fit = dataframe_2
        sp.initial_model = initial_model
        sp.df_pre = _df
        sp.df_metric = _df
        return {"sp": sp, "tmpdir": str(tmpdir)}


@pytest.fixture(name="class_spectrafit_fit")
def class_spectrafit_fixture_fit(
    dataframe: pd.DataFrame, x_column: str, y_column: str
) -> SpectraFitNotebook:
    """Create a SpectraFitNotebook object."""
    with TemporaryDirectory() as tmpdir:
        return SpectraFitNotebook(
            df=dataframe,
            x_column=x_column,
            y_column=y_column,
            fname="test",
            folder=str(tmpdir),
        )


def test_dataframe_display(dataframe: pd.DataFrame) -> None:
    """Test the DataFrameDisplay class."""
    DataFrameDisplay().df_display(df=dataframe, mode="regular")
    DataFrameDisplay().df_display(df=dataframe, mode="markdown")
    DataFrameDisplay().df_display(df=dataframe, mode="interactive")
    DataFrameDisplay().df_display(df=dataframe, mode="dtale")

    with pytest.raises(ValueError) as excinfo:
        DataFrameDisplay().df_display(df=dataframe, mode="wrong")

    assert "Invalid mode: wrong." in str(excinfo.value)


@pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
@pytest.mark.webtest
class TestDataFramePlot:
    """Test the DataFramePlot class."""

    def test_dataframe_plot_1(self, dataframe: pd.DataFrame) -> None:
        """Test single plot with one y-column."""
        pp = DataFramePlot()
        with mock.patch(__plotly_io_show__) as mock_show:
            pp.plot_dataframe(
                args_plot=PlotAPI(x="Energy", y="Noisy", title="Test"), df=dataframe
            )
            mock_show.assert_called_once()

    def test_dataframe_plot_2(self, dataframe: pd.DataFrame) -> None:
        """Test single plot with two y-column."""
        pp = DataFramePlot()
        with mock.patch(__plotly_io_show__) as mock_show:
            pp.plot_dataframe(
                args_plot=PlotAPI(x="Energy", y=["Intensity", "Noisy"], title="Test"),
                df=dataframe,
            )
            mock_show.assert_called_once()

    def test_dataframe_plot_3(self, dataframe_2: pd.DataFrame) -> None:
        """Test douple plot."""
        pp = DataFramePlot()
        with mock.patch(__plotly_io_show__) as mock_show:
            pp.plot_2dataframes(
                args_plot=PlotAPI(x="energy", y="intensity", title="Test"),
                df_1=dataframe_2,
                df_2=dataframe_2,
            )
            mock_show.assert_called_once()

    def test_dataframe_plot_4(self, dataframe_2: pd.DataFrame) -> None:
        """Test double plot with residual."""
        pp = DataFramePlot()
        with mock.patch(__plotly_io_show__) as mock_show:
            pp.plot_2dataframes(
                args_plot=PlotAPI(x="energy", y="intensity", title="Test"),
                df_1=dataframe_2,
            )
            mock_show.assert_called_once()


def test_dataframe_display_all(dataframe: pd.DataFrame) -> None:
    """Test the DataFrameDisplay class."""
    DataFrameDisplay().df_display(df=dataframe, mode="regular")
    DataFrameDisplay().df_display(df=dataframe, mode="markdown")
    DataFrameDisplay().df_display(df=dataframe, mode="interactive")
    DataFrameDisplay().df_display(df=dataframe, mode="dtale")

    with pytest.raises(ValueError) as excinfo:
        DataFrameDisplay().df_display(df=dataframe, mode="wrong")

    assert "Invalid mode: wrong." in str(excinfo.value)


class TestExportResults:
    """Test of the Export Results class."""

    def test_export_df(self, dataframe: pd.DataFrame) -> None:
        """Test the export function."""
        with TemporaryDirectory() as tmpdir:
            fname = "test"
            prefix = "test"
            suffix = "csv"
            folder = str(tmpdir)
            ExportResults().export_df(
                df=dataframe,
                args=FnameAPI(fname=fname, prefix=prefix, suffix=suffix, folder=folder),
            )
            assert Path(folder).joinpath(f"{prefix}_{fname}.{suffix}").exists()

    def test_export_report(self, dataframe: pd.DataFrame) -> None:
        """Test the export function."""
        with TemporaryDirectory() as tmpdir:
            fname = "test"
            prefix = "test"
            suffix = "lock"
            folder = str(tmpdir)
            ExportResults().export_report(
                report=dataframe.to_dict(orient="list"),
                args=FnameAPI(fname=fname, prefix=prefix, suffix=suffix, folder=folder),
            )
            assert Path(folder).joinpath(f"{prefix}_{fname}.{suffix}").exists()

    def test_static_fname(self) -> None:
        """Test the static function of generating PathPosixs."""
        assert isinstance(ExportResults.fname2Path(fname="test", suffix="csv"), Path)
        assert isinstance(
            ExportResults.fname2Path(
                fname="test", suffix="csv", folder="tmp", prefix="prefix"
            ),
            Path,
        )


class TestExportReport:
    """Test the ExportReport class."""

    def test_input(self, export_report: Dict[str, Any]) -> None:
        """Test the input function."""
        assert isinstance(
            ExportReport(**export_report).make_input_contribution, InputAPI
        )

    def test_solver(self, export_report: Dict[str, Any]) -> None:
        """Test the solver function."""
        assert isinstance(
            ExportReport(**export_report).make_solver_contribution, SolverAPI
        )

    def test_output(self, export_report: Dict[str, Any]) -> None:
        """Test the output function."""
        assert isinstance(
            ExportReport(**export_report).make_output_contribution, OutputAPI
        )


class TestSpectraFitNotebook:
    """Test the SpectraFitNotebook class."""

    def test_init(
        self,
        dataframe: pd.DataFrame,
        x_column: str,
        y_column: str,
        y_columns: List[str],
    ) -> None:
        """Test the initialization of the class."""
        assert isinstance(
            SpectraFitNotebook(df=dataframe, x_column=x_column, y_column=y_column),
            SpectraFitNotebook,
        )
        assert isinstance(
            SpectraFitNotebook(df=dataframe, x_column=x_column, y_column=y_columns),
            SpectraFitNotebook,
        )

    def test_init_fail(self, dataframe: pd.DataFrame, x_column: str) -> None:
        """Test the initialization of the class."""
        with pytest.raises(ValueError) as excinfo:
            SpectraFitNotebook(
                df=dataframe[[x_column]], x_column=x_column, y_column="wrong"
            )

        assert "The dataframe must have 2 or more columns." in str(excinfo.value)

    def test_pre_process(
        self, dataframe: pd.DataFrame, x_column: str, y_column: str
    ) -> None:
        """Test the pre_process function."""
        sp = SpectraFitNotebook(df=dataframe, x_column=x_column, y_column=y_column)
        sp.pre_process
        sp.df_fit = dataframe
        assert isinstance(sp.return_df_org, pd.DataFrame)
        assert isinstance(sp.return_df_pre, pd.DataFrame)
        assert isinstance(sp.return_df_fit, pd.DataFrame)
        assert isinstance(sp.return_df, pd.DataFrame)
        assert isinstance(sp.return_pre_statistic, dict)

    def test_export_df(self, class_spectrafit: Dict[Any, Any]) -> None:
        """Test the export_df function."""
        class_spectrafit["sp"].export_df_act
        class_spectrafit["sp"].export_df_fit
        class_spectrafit["sp"].export_df_org
        class_spectrafit["sp"].export_df_pre
        class_spectrafit["sp"].export_df_metric
        assert ExportResults.fname2Path(
            folder=class_spectrafit["tmpdir"], fname="test", prefix="act", suffix="csv"
        ).exists()
        assert ExportResults.fname2Path(
            folder=class_spectrafit["tmpdir"], fname="test", prefix="fit", suffix="csv"
        ).exists()
        assert ExportResults.fname2Path(
            folder=class_spectrafit["tmpdir"], fname="test", prefix="org", suffix="csv"
        ).exists()
        assert ExportResults.fname2Path(
            folder=class_spectrafit["tmpdir"], fname="test", prefix="pre", suffix="csv"
        ).exists()
        assert ExportResults.fname2Path(
            folder=class_spectrafit["tmpdir"],
            fname="test",
            prefix="metric",
            suffix="csv",
        ).exists()

    @pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
    @pytest.mark.webtest
    def test_plot_org(
        self,
        class_spectrafit: Dict[Any, Any],
    ) -> None:
        """Test the plot function."""
        with mock.patch(__plotly_io_show__) as mock_show:
            class_spectrafit["sp"].plot_original_df
            mock_show.assert_called_once()

    @pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
    @pytest.mark.webtest
    def test_plot_current(
        self,
        class_spectrafit: Dict[Any, Any],
    ) -> None:
        """Test the plot function."""
        with mock.patch(__plotly_io_show__) as mock_show:
            class_spectrafit["sp"].plot_current_df
            mock_show.assert_called_once()

    @pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
    @pytest.mark.webtest
    def test_plot_preprocess(
        self,
        class_spectrafit: Dict[Any, Any],
    ) -> None:
        """Test the plot function."""
        with mock.patch(__plotly_io_show__) as mock_show:
            class_spectrafit["sp"].plot_preprocessed_df
            mock_show.assert_called_once()

    @pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
    @pytest.mark.webtest
    def test_plot_fit(
        self,
        class_spectrafit: Dict[Any, Any],
    ) -> None:
        """Test the plot function."""
        with mock.patch(__plotly_io_show__) as mock_show:
            class_spectrafit["sp"].plot_fit_df
            mock_show.assert_called_once()

    def test_display(
        self,
        class_spectrafit: Dict[Any, Any],
    ) -> None:
        """Test the display function."""
        class_spectrafit["sp"].display_preprocessed_df
        class_spectrafit["sp"].display_original_df
        class_spectrafit["sp"].display_current_df
        class_spectrafit["sp"].display_fit_df

    def test_generate_report(
        self,
        class_spectrafit_fit: SpectraFitNotebook,
        dataframe_2: pd.DataFrame,
        initial_model: List[Dict[str, Dict[str, Dict[str, Any]]]],
        args_out: Dict[str, Any],
    ) -> None:
        """Test the generate_report function."""
        sp = class_spectrafit_fit
        sp.initial_model = initial_model
        sp.args = args_out
        sp.df_fit = dataframe_2
        sp.generate_report

        assert ExportResults.fname2Path(
            folder=sp.export_args_out.folder,
            fname="test",
            suffix="lock",
        ).exists()

    def test_fit(
        self,
        class_spectrafit_fit: SpectraFitNotebook,
        initial_model: List[Dict[str, Dict[str, Dict[str, Any]]]],
    ) -> None:
        """Test the fit function."""
        sp = class_spectrafit_fit

        with mock.patch(__plotly_io_show__) as mock_show:
            sp.solver_model(
                initial_model=initial_model,
                show_plot=True,
                show_df=True,
                show_metric=False,
            )
            mock_show.assert_called_once()

    def test_metric(
        self,
        class_spectrafit_fit: SpectraFitNotebook,
        initial_model: List[Dict[str, Dict[str, Dict[str, Any]]]],
    ) -> None:
        """Test the metric plot function."""
        sp = class_spectrafit_fit

        with mock.patch(__plotly_io_show__) as mock_show:
            sp.solver_model(
                initial_model=initial_model,
                show_plot=False,
                show_df=True,
                show_metric=True,
            )
            mock_show.assert_called_once()

    def test_conv_1(
        self,
        class_spectrafit_fit: SpectraFitNotebook,
        initial_model: List[Dict[str, Dict[str, Dict[str, Any]]]],
    ) -> None:
        """Test conf interval via bool."""
        sp = class_spectrafit_fit

        with mock.patch(__plotly_io_show__) as mock_show:
            sp.solver_model(
                initial_model=initial_model,
                show_plot=False,
                show_df=True,
                show_metric=True,
                conf_interval=True,
            )
            mock_show.assert_called_once()

    def test_conv_2(
        self,
        class_spectrafit_fit: SpectraFitNotebook,
        initial_model: List[Dict[str, Dict[str, Dict[str, Any]]]],
    ) -> None:
        """Test conf interval via bool."""
        sp = class_spectrafit_fit

        with mock.patch(__plotly_io_show__) as mock_show:
            sp.solver_model(
                initial_model=initial_model,
                show_plot=False,
                show_df=True,
                show_metric=True,
                conf_interval={},
            )

            mock_show.assert_called_once()



"""Reference model for the API of the Jupyter Notebook interface."""


from typing import List
from typing import Optional
from typing import Tuple
from typing import Union

from _plotly_utils.colors.carto import Burg
from _plotly_utils.colors.carto import Purp_r
from _plotly_utils.colors.carto import Teal_r
from _plotly_utils.colors.qualitative import Plotly as PlotlyColors
from pydantic import BaseModel
from pydantic import Field
from pydantic import validator


class XAxisAPI(BaseModel):
    """Defintion of the X-Axis of the plotly figure."""

    name: str = Field(default="Energy", description="Name of the x-axis of the plot.")
    unit: Optional[str] = Field(
        default="eV", description="Name of the x-axis units of the plot."
    )


class YAxisAPI(BaseModel):
    """Defintion of the Y-Axis of the plotly figure."""

    name: str = Field(
        default="Intensity", description="Name of the y-axis of the plot."
    )
    unit: Optional[str] = Field(
        default="a.u.", description="Name of the y-axis units of the plot."
    )


class ResidualAPI(BaseModel):
    """Definition of the residual plot (Y-Axis) of the plotly figure."""

    name: str = Field(
        default="Residuals", description="Name of the residual-axis of the plot."
    )
    unit: Optional[str] = Field(
        default="a.u.", description="Name of the residual-axis units of the plot."
    )


class MetricAPI(BaseModel):
    """Definition of the residual plot (Y-Axis) of the plotly figure."""

    name_0: str = Field(
        default="Metrics", description="Name of the first metrics-axis of the plot."
    )
    unit_0: Optional[str] = Field(
        default="a.u.", description="Name of the first metrics-axis units of the plot."
    )
    name_1: str = Field(
        default="Metrics", description="Name of the second metrics-axis of the plot."
    )
    unit_1: Optional[str] = Field(
        default="a.u.", description="Name of the second metrics-axis units of the plot."
    )


class RunAPI(BaseModel):
    """Definition of the residual plot (Y-Axis) of the plotly figure."""

    name: str = Field(default="Run", description="Name of the Run-axis of the plot.")
    unit: Optional[str] = Field(
        default="#", description="Name of the run-axis units of the plot."
    )


class FontAPI(BaseModel):
    """Definition of the used font of the plotly figure."""

    family: str = Field(
        default="Open Sans, monospace", description="Font family of the plot."
    )
    size: int = Field(default=12, description="Font size of the plot.")
    color: str = Field(default="black", description="Font color of the plot.")


class LegendAPI(BaseModel):
    """Definition of the legend of the plotly figure."""

    orientation: str = Field(default="h", description="Orientation of the legend.")
    yanchor: str = Field(default="bottom", description="Y anchor of the legend.")
    y: float = Field(default=1.02, description="Y position of the legend.")
    xanchor: str = Field(default="right", description="X anchor of the legend.")
    x: float = Field(default=1, description="X position of the legend.")


class GridAPI(BaseModel):
    """Definition of the grid of the plotly figure."""

    show: bool = Field(default=True, description="Show grid lines.")
    ticks: str = Field(default="outside", description="Show grid ticks.")
    dash: str = Field(default="dot", description="Show grid dashes.")


class ColorAPI(BaseModel):
    """Definition of the colors of the plotly figure."""

    intensity: str = Field(
        default=PlotlyColors[0], description="Color of the spectrum-intensity."
    )
    residual: str = Field(
        default=PlotlyColors[1], description="Color of the residuals."
    )
    fit: str = Field(default=PlotlyColors[5], description="Color of the fit.")
    components: str = Field(
        default=PlotlyColors[6], description="Color of the components, mainly peaks."
    )
    # Merging two color list to onw list with switchin the order of the colors
    bars: List[str] = Field(
        default=[i for j in zip(Teal_r, Purp_r) for i in j],
        description="Color of the bar plot of the metrics.",
    )
    lines: List[str] = Field(default=Burg, description="Color of the lines plot.")
    paper: str = Field(default="white", description="Color of the paper.")
    plot: str = Field(default="white", description="Color of the plot.")
    color: str = Field(default="black", description="Color of the text.")
    grid: str = Field(default="lightgrey", description="Color of the grid.")
    line: str = Field(default="black", description="Color of the bottom and side line.")
    zero_line: str = Field(default="grey", description="Color of the zero line.")
    ticks: str = Field(default="black", description="Color of the ticks.")
    font: str = Field(default="black", description="Font color of the plot.")

    @validator(
        "page",
        "layout",
        "grid",
        "line",
        "zero_line",
        "ticks",
        "font",
        check_fields=False,
    )
    @classmethod
    def transparent_rgb(cls, v: str) -> str:
        """Convert string to transparent RGB color.

        Args:
            v (str): One of the key-words of the validator decorator.

        Returns:
            str: Translate the word `transparent` to the rgb value `rgba(0,0,0,0)`.
        """
        return "rgba(0,0,0,0)" if "transparent" in v.lower() else v


class PlotAPI(BaseModel):
    """Definition of the plotly figure."""

    x: str = Field(..., description="Name of the x column to plot.")
    y: Union[str, List[str]] = Field(
        ..., description="List of the names of the y columns to plot."
    )
    title: Optional[str] = Field(None, description="Title of the plot.")
    xaxis_title: XAxisAPI = XAxisAPI()
    yaxis_title: YAxisAPI = YAxisAPI()
    residual_title: ResidualAPI = ResidualAPI()
    metric_title: MetricAPI = MetricAPI()
    run_title: RunAPI = RunAPI()
    legend_title: str = Field(default="Spectra", description="Title of the legend.")
    show_legend: bool = Field(default=True, description="Show legend.")
    legend: LegendAPI = LegendAPI()
    font: FontAPI = FontAPI()
    minor_ticks: bool = Field(default=True, description="Show minor ticks.")
    color: ColorAPI = ColorAPI()
    grid: GridAPI = GridAPI()
    size: Tuple[int, Tuple[int, int]] = Field(
        default=(800, (600, 300)), description="Size of the fit- and metric-plot."
    )


class FnameAPI(BaseModel):
    """Definition of the file name."""

    fname: str = Field(..., description="Name of the file to save.")
    suffix: str = Field(..., description="Suffix of the file to save.")
    prefix: Optional[str] = Field(
        default=None, description="Prefix of the file to save."
    )
    folder: Optional[str] = Field(default=None, description="Folder to save the file.")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['spectrafit',
 'spectrafit.api',
 'spectrafit.api.test',
 'spectrafit.app',
 'spectrafit.app.test',
 'spectrafit.plugins',
 'spectrafit.plugins.test',
 'spectrafit.test',
 'spectrafit.utilities',
 'spectrafit.utilities.test']

package_data = \
{'': ['*'], 'spectrafit.test': ['export/*', 'import/*', 'scripts/*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'art>=5.7,<6.0',
 'emcee>=3.1.2,<4.0.0',
 'lmfit>=1.0.3,<2.0.0',
 'numdifftools>=0.9.39,<0.10.0',
 'openpyxl>=3.0.9,<4.0.0',
 'pydantic>=1.10.1,<2.0.0',
 'scikit-learn>=1.0.2,<2.0.0',
 'seaborn>=0.11.2,<0.13.0',
 'tabulate>=0.8.10,<0.10.0',
 'tomli-w>=1.0.0,<2.0.0',
 'tomli>=2.0.1,<3.0.0',
 'tqdm>=4.64.0,<5.0.0']

extras_require = \
{':python_version < "3.8"': ['pandas==1.3.5', 'numpy==1.21.6'],
 ':python_version >= "3.8"': ['pandas>=1.5.0,<2.0.0', 'numpy>=1.23.4,<2.0.0'],
 'jupyter': ['jupyterlab>=3.4.5,<4.0.0',
             'plotly>=5.11.0,<6.0.0',
             'itables>=1.3.0,<2.0.0',
             'kaleido==0.2.1',
             'dtale>=2.8.1,<3.0.0']}

entry_points = \
{'console_scripts': ['spectrafit = spectrafit.spectrafit:command_line_runner',
                     'spectrafit-converter = '
                     'spectrafit.plugins.converter:command_line_runner',
                     'spectrafit-jupyter = spectrafit.app.app:jupyter']}

setup_kwargs = {
    'name': 'spectrafit',
    'version': '0.12.5',
    'description': 'Fast fitting of 2D-Spectra with established routines',
    'long_description': '[![CI - Python Package](https://github.com/Anselmoo/spectrafit/actions/workflows/python-ci.yml/badge.svg?branch=main)](https://github.com/Anselmoo/spectrafit/actions/workflows/python-ci.yml)\n[![codecov](https://codecov.io/gh/Anselmoo/spectrafit/branch/main/graph/badge.svg?token=pNIMKwWsO2)](https://codecov.io/gh/Anselmoo/spectrafit)\n[![PyPI](https://img.shields.io/pypi/v/spectrafit?logo=PyPi&logoColor=yellow)](https://pypi.org/project/spectrafit/)\n[![Conda](https://img.shields.io/conda/v/conda-forge/spectrafit?label=Anaconda.org&logo=anaconda)](https://anaconda.org/conda-forge/spectrafit)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/spectrafit?color=gree&logo=Python&logoColor=yellow)](https://pypi.org/project/spectrafit/)\n[![pre-commit.ci status](https://results.pre-commit.ci/badge/github/Anselmoo/spectrafit/main.svg)](https://results.pre-commit.ci/latest/github/Anselmoo/spectrafit/main)\n\n<p align="center">\n<img src="https://github.com/Anselmoo/spectrafit/blob/c5f7ee05e5610fb8ef4e237a88f62977b6f832e5/docs/images/spectrafit_synopsis.png?raw=true">\n</p>\n\n# SpectraFit\n\n`SpectraFit` is a tool for quick data fitting based on the regular\nexpression of distribution and linear functions via command line or\n[Jupyter Notebook](https://jupyter.org). Furthermore, it can be also used as\na module in existing python code. A previous version of `SpectraFit` was used\nfor the following publication:\n\n- [Measurement of the Ligand Field Spectra of Ferrous and Ferric Iron Chlorides Using 2p3d RIXS](https://pubs.acs.org/doi/abs/10.1021/acs.inorgchem.7b00940)\n\nNow, it is completely rewritten and is more flexible. It is supporting all\ncommon ASCII-data formats and it runs on `Linux`, `Windows`, and `MacOS`.\n\n## Scope\n\n- Fitting of 2D data, also with multiple columns as _global fitting_\n- Using established and advanced solver methods\n- Extensibility of the fitting function\n- Guarantee traceability of the fitting results\n- Saving all results in a _SQL-like-format_ (`CSV`) for publications\n- Saving all results in a _NoSQL-like-format_ (`JSON`) for project management\n- Having an API interface for Graph-databases\n\n## Installation\n\nvia pip:\n\n```terminal\npip install spectrafit\n\n# with suppot for Jupyter Notebook\n\npip install spectrafit[jupyter]\n\n# Upgrade\n\npip install spectrafit --upgrade\n```\n\nvia conda:\n\n```terminal\nconda install -c conda-forge spectrafit\n```\n\n## Usage\n\n`SpectraFit` needs as command line tool only two things:\n\n1. The reference data, which should be fitted.\n2. The input file, which contains the initial model.\n\nAs model files [json](https://en.wikipedia.org/wiki/JSON),\n[toml](https://en.wikipedia.org/wiki/TOML), and\n[yaml](https://en.wikipedia.org/wiki/YAML) are supported. By making use of the\npython `**kwargs` feature, the input file can call most of the following\nfunctions of [LMFIT](https://lmfit.github.io/lmfit-py/index.html). LMFIT is the\nworkhorse for the fit optimization, which is macro wrapper based on:\n\n1. [NumPy](https://www.numpy.org/)\n2. [SciPy](https://www.scipy.org/)\n3. [uncertainties](https://pythonhosted.org/uncertainties/)\n\nIn case of `SpectraFit`, we have further extend the package by:\n\n1. [Pandas](https://pandas.pydata.org/)\n2. [statsmodels](https://www.statsmodels.org/stable/index.html)\n3. [numdifftools](https://github.com/pbrod/numdifftools)\n4. [Matplotlib](https://matplotlib.org/) in combination with\n   [Seaborn](https://seaborn.pydata.org/)\n\n```terminal\nspectrafit data_file.txt -i input_file.json\n```\n\n```terminal\nusage: spectrafit [-h] [-o OUTFILE] [-i INPUT] [-ov] [-e0 ENERGY_START]\n                  [-e1 ENERGY_STOP] [-s SMOOTH] [-sh SHIFT] [-c COLUMN COLUMN]\n                  [-sep {       ,,,;,:,|, ,s+}] [-dec {.,,}] [-hd HEADER]\n                  [-g {0,1,2}] [-auto] [-np] [-v] [-vb {0,1,2}]\n                  infile\n\nFast Fitting Program for ascii txt files.\n\npositional arguments:\n  infile                Filename of the spectra data\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -o OUTFILE, --outfile OUTFILE\n                        Filename for the export, default to set to\n                        \'spectrafit_results\'.\n  -i INPUT, --input INPUT\n                        Filename for the input parameter, default to set to\n                        \'fitting_input.toml\'.Supported fileformats are:\n                        \'*.json\', \'*.yml\', \'*.yaml\', and \'*.toml\'\n  -ov, --oversampling   Oversampling the spectra by using factor of 5;\n                        default to False.\n  -e0 ENERGY_START, --energy_start ENERGY_START\n                        Starting energy in eV; default to start of energy.\n  -e1 ENERGY_STOP, --energy_stop ENERGY_STOP\n                        Ending energy in eV; default to end of energy.\n  -s SMOOTH, --smooth SMOOTH\n                        Number of smooth points for lmfit; default to 0.\n  -sh SHIFT, --shift SHIFT\n                        Constant applied energy shift; default to 0.0.\n  -c COLUMN COLUMN, --column COLUMN COLUMN\n                        Selected columns for the energy- and intensity-values;\n                        default to \'0\' for energy (x-axis) and \'1\' for intensity\n                        (y-axis). In case of working with header, the column\n                        should be set to the column names as \'str\'; default\n                        to 0 and 1.\n  -sep { ,,,;,:,|, ,s+}, --separator { ,,,;,:,|, ,s+}\n                        Redefine the type of separator; default to \' \'.\n  -dec {.,,}, --decimal {.,,}\n                        Type of decimal separator; default to \'.\'.\n  -hd HEADER, --header HEADER\n                        Selected the header for the dataframe; default to None.\n  -cm COMMENT, --comment COMMENT\n                        Lines with comment characters like \'#\' should not be\n                        parsed; default to None.\n  -g {0,1,2}, --global_ {0,1,2}\n                        Perform a global fit over the complete dataframe. The\n                        options are \'0\' for classic fit (default). The\n                        option \'1\' for global fitting with auto-definition\n                        of the peaks depending on the column size and \'2\'\n                        for self-defined global fitting routines.\n  -auto, --autopeak     Auto detection of peaks in the spectra based on `SciPy`.\n                        The position, height, and width are used as estimation\n                        for the `Gaussian` models.The default option is \'False\'\n                        for  manual peak definition.\n  -np, --noplot         No plotting the spectra and the fit of `SpectraFit`.\n  -v, --version         Display the current version of `SpectraFit`.\n  -vb {0,1,2}, --verbose {0,1,2}\n                        Display the initial configuration parameters and fit\n                        results, as a table \'1\', as a dictionary \'2\', or not in\n                        the terminal \'0\'. The default option is set to 1 for\n                        table `printout`.\n```\n\n### Jupyter Notebook\n\nOpen the `Jupyter Notebook` and run the following code:\n\n```terminal\nspectrafit-jupyter\n```\n\nor via Docker Image:\n\n```terminal\ndocker pull ghcr.io/anselmoo/spectrafit:latest\ndocker run -it -p 8888:8888 spectrafit:latest\n```\n\nor just:\n\n```terminal\ndocker run -p 8888:8888 ghcr.io/anselmoo/spectrafit:latest\n```\n\nNext define your initial model and the reference data:\n\n```python\nfrom spectrafit.plugins.notebook import SpectraFitNotebook\nimport pandas as pd\n\ndf = pd.read_csv(\n    "https://raw.githubusercontent.com/Anselmoo/spectrafit/main/Examples/data.csv"\n)\n\ninitial_model = [\n    {\n        "pseudovoigt": {\n            "amplitude": {"max": 2, "min": 0, "vary": True, "value": 1},\n            "center": {"max": 2, "min": -2, "vary": True, "value": 0},\n            "fwhmg": {"max": 0.4, "min": 0.1, "vary": True, "value": 0.21},\n            "fwhml": {"max": 0.4, "min": 0.1, "vary": True, "value": 0.21},\n        }\n    },\n    {\n        "pseudovoigt": {\n            "amplitude": {"max": 2, "min": 0, "vary": True, "value": 1},\n            "center": {"max": 2, "min": -2, "vary": True, "value": 1},\n            "fwhmg": {"max": 0.4, "min": 0.1, "vary": True, "value": 0.21},\n            "fwhml": {"max": 0.4, "min": 0.1, "vary": True, "value": 0.21},\n        }\n    },\n    {\n        "pseudovoigt": {\n            "amplitude": {"max": 2, "min": 0, "vary": True, "value": 1},\n            "center": {"max": 2, "min": -2, "vary": True, "value": 1},\n            "fwhmg": {"max": 0.4, "min": 0.1, "vary": True, "value": 0.21},\n            "fwhml": {"max": 0.4, "min": 0.1, "vary": True, "value": 0.21},\n        }\n    },\n]\nspf = SpectraFitNotebook(df=df, x_column="Energy", y_column="Noisy")\nspf.solver_model(initial_model)\n```\n\nWhich results in the following output:\n\n![img_jupyter](https://github.com/Anselmoo/spectrafit/blob/8962a277b0c3d2aa05970617f0ac323a07de2fec/docs/images/jupyter_plot.png?raw=true)\n\n## Documentation\n\nPlease see the [extended documentation](https://anselmoo.github.io/spectrafit/)\nfor the full usage of `SpectraFit`.\n',
    'author': 'Anselm Hahn',
    'author_email': 'anselm.hahn@gmail.com',
    'maintainer': 'Anselm Hahn',
    'maintainer_email': 'anselm.hahn@gmail.com',
    'url': 'https://pypi.org/project/spectrafit/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<3.11',
}


setup(**setup_kwargs)
